@@ -266,41 +266,24 @@ class ArgumentParser(argparse.ArgumentParser):
     """
 
     def __init__(self,
-        prog=None,
-        usage=None,
-        description=None,
-        epilog=None,
-        version=None,
-        parents=[],
-        formatter_class=argparse.HelpFormatter,
-        prefix_chars='-',
-        fromfile_prefix_chars=None,
-        argument_default=None,
-        conflict_handler='error',
-        add_help=True,
-
         add_config_file_help=True,
         add_env_var_help=True,
-
         auto_env_var_prefix=None,
-
         default_config_files=[],
         ignore_unknown_config_file_keys=False,
         config_file_parser_class=DefaultConfigFileParser,
-
         args_for_setting_config_path=[],
         config_arg_is_required=False,
         config_arg_help_message="config file path",
-
         args_for_writing_out_config_file=[],
         write_out_config_file_arg_help_message="takes the current command line "
             "args and writes them out to a config file at the given path, then "
             "exits",
-        allow_abbrev=True,  # new in python 3.5
+        **kwargs
         ):
 
-        """Supports all the same args as the argparse.ArgumentParser
-        constructor, as well as the following additional args.
+        """Supports args of the argparse.ArgumentParser constructor
+        as **kwargs, as well as the following additional args.
 
         Additional Args:
             add_config_file_help: Whether to add a description of config file
@@ -347,26 +330,14 @@ class ArgumentParser(argparse.ArgumentParser):
                 (eg. ["-w", "--write-out-config-file"]). Default: []
             write_out_config_file_arg_help_message: The help message to use for
                 the args in args_for_writing_out_config_file.
-            allow_abbrev: Allows long options to be abbreviated if the
-                abbreviation is unambiguous. Default: True
         """
         self._add_config_file_help = add_config_file_help
         self._add_env_var_help = add_env_var_help
         self._auto_env_var_prefix = auto_env_var_prefix
 
-        # extract kwargs that can be passed to the super constructor
-        kwargs_for_super = dict((k, v) for k, v in locals().items() if k in [
-            "prog", "usage", "description", "epilog", "version", "parents",
-            "formatter_class", "prefix_chars", "fromfile_prefix_chars",
-            "argument_default", "conflict_handler", "add_help"])
-        if sys.version_info >= (3, 3) and "version" in kwargs_for_super:
-            del kwargs_for_super["version"]  # version arg deprecated in v3.3
-        if sys.version_info >= (3, 5):
-            kwargs_for_super["allow_abbrev"] = allow_abbrev  # new option in v3.5
+        argparse.ArgumentParser.__init__(self, **kwargs)
 
-        argparse.ArgumentParser.__init__(self, **kwargs_for_super)
-
-        # parse the additionial args
+        # parse the additional args
         if config_file_parser_class is None:
             self._config_file_parser = DefaultConfigFileParser()
         else:
@@ -451,6 +422,7 @@ class ArgumentParser(argparse.ArgumentParser):
                 config_file_keys = self.get_possible_config_keys(a)
                 if config_file_keys and not (a.env_var or a.is_positional_arg
                     or a.is_config_file_arg or a.is_write_out_config_file_arg or
+                    isinstance(a, argparse._VersionAction) or
                     isinstance(a, argparse._HelpAction)):
                     stripped_config_file_key = config_file_keys[0].strip(
                         self.prefix_chars)
@@ -464,11 +436,16 @@ class ArgumentParser(argparse.ArgumentParser):
                 and not already_on_command_line(args, a.option_strings)]
         for action in actions_with_env_var_values:
             key = action.env_var
-            value = env_vars[key]  # TODO parse env var values here to allow lists?
+            value = env_vars[key]
+            # Make list-string into list.
+            if action.nargs or isinstance(action, argparse._AppendAction):
+                element_capture = re.match('\[(.*)\]', value)
+                if element_capture:
+                    value = [val.strip() for val in element_capture.group(1).split(',') if val.strip()]
             env_var_args += self.convert_item_to_command_line_arg(
                 action, key, value)
 
-        args = env_var_args + args
+        args = args + env_var_args
 
         if env_var_args:
             self._source_to_settings[_ENV_VAR_SOURCE_KEY] = OrderedDict(
@@ -488,7 +465,7 @@ class ArgumentParser(argparse.ArgumentParser):
 
         # open the config file(s)
         config_streams = []
-        if config_file_contents:
+        if config_file_contents is not None:
             stream = StringIO(config_file_contents)
             stream.name = "method arg"
             config_streams = [stream]
@@ -527,7 +504,7 @@ class ArgumentParser(argparse.ArgumentParser):
                         self._source_to_settings[source_key] = OrderedDict()
                     self._source_to_settings[source_key][key] = (action, value)
 
-            args = config_args + args
+            args = args + config_args
 
 
         # save default settings for use by print_values()
@@ -555,35 +532,44 @@ class ArgumentParser(argparse.ArgumentParser):
         namespace, unknown_args = argparse.ArgumentParser.parse_known_args(
             self, args=args, namespace=namespace)
         # handle any args that have is_write_out_config_file_arg set to true
-        user_write_out_config_file_arg_actions = [a for a in self._actions
-            if getattr(a, "is_write_out_config_file_arg", False)]
-        if user_write_out_config_file_arg_actions:
-            output_file_paths = []
-            for action in user_write_out_config_file_arg_actions:
-                # check if the user specified this arg on the commandline
-                output_file_path = getattr(namespace, action.dest, None)
-                if output_file_path:
-                    # validate the output file path
-                    try:
-                        with open(output_file_path, "w") as output_file:
-                            output_file_paths.append(output_file_path)
-                    except IOError as e:
-                        raise ValueError("Couldn't open %s for writing: %s" % (
-                            output_file_path, e))
-
-            if output_file_paths:
-                # generate the config file contents
-                config_items = self.get_items_for_config_file_output(
-                    self._source_to_settings, namespace)
-                file_contents = self._config_file_parser.serialize(config_items)
-                for output_file_path in output_file_paths:
-                    with open(output_file_path, "w") as output_file:
-                        output_file.write(file_contents)
-                if len(output_file_paths) == 1:
-                    output_file_paths = output_file_paths[0]
-                self.exit(0, "Wrote config file to " + str(output_file_paths))
+        # check if the user specified this arg on the commandline
+        output_file_paths = [getattr(namespace, a.dest, None) for a in self._actions
+                             if getattr(a, "is_write_out_config_file_arg", False)]
+        output_file_paths = [a for a in output_file_paths if a is not None]
+        self.write_config_file(namespace, output_file_paths, exit_after=True)
         return namespace, unknown_args
 
+    def write_config_file(self, parsed_namespace, output_file_paths, exit_after=False):
+        """Write the given settings to output files.
+
+        Args:
+            parsed_namespace: namespace object created within parse_known_args()
+            output_file_paths: any number of file paths to write the config to
+            exit_after: whether to exit the program after writing the config files
+        """
+        for output_file_path in output_file_paths:
+            # validate the output file path
+            try:
+                with open(output_file_path, "w") as output_file:
+                    pass
+            except IOError as e:
+                raise ValueError("Couldn't open %s for writing: %s" % (
+                    output_file_path, e))
+        if output_file_paths:
+            # generate the config file contents
+            config_items = self.get_items_for_config_file_output(
+                self._source_to_settings, parsed_namespace)
+            file_contents = self._config_file_parser.serialize(config_items)
+            for output_file_path in output_file_paths:
+                with open(output_file_path, "w") as output_file:
+                    output_file.write(file_contents)
+            message = "Wrote config file to " + ", ".join(output_file_paths)
+            if exit_after:
+                self.exit(0, message)
+            else:
+                print(message)
+
+
     def get_command_line_key_for_unknown_config_file_setting(self, key):
         """Compute a commandline arg key to be used for a config file setting
         that doesn't correspond to any defined configargparse arg (and so
@@ -622,10 +608,6 @@ class ArgumentParser(argparse.ArgumentParser):
                         if value is not None:
                             if isinstance(value, bool):
                                 value = str(value).lower()
-                            elif callable(action.type):
-                                found = [i for i in range(0, len(existing_command_line_args)-1) if existing_command_line_args[i] in config_file_keys]
-                                if found:
-                                    value = existing_command_line_args[found[-1] + 1]
                             config_file_items[config_file_keys[0]] = value
 
             elif source == _ENV_VAR_SOURCE_KEY:
@@ -659,6 +641,7 @@ class ArgumentParser(argparse.ArgumentParser):
             value: parsed value of type string or list
         """
         args = []
+
         if action is None:
             command_line_key = \
                 self.get_command_line_key_for_unknown_config_file_setting(key)
@@ -667,14 +650,14 @@ class ArgumentParser(argparse.ArgumentParser):
 
         # handle boolean value
         if action is not None and isinstance(action, ACTION_TYPES_THAT_DONT_NEED_A_VALUE):
-            if value.lower() in ("true", "yes"):
+            if value.lower() in ("true", "yes", "1"):
                 args.append( command_line_key )
-            elif value.lower() in ("false", "no"):
+            elif value.lower() in ("false", "no", "0"):
                 # don't append when set to "false" / "no"
                 pass
             else:
                 self.error("Unexpected value for %s: '%s'. Expecting 'true', "
-                           "'false', 'yes', or 'no'" % (key, value))
+                           "'false', 'yes', 'no', '1' or '0'" % (key, value))
         elif isinstance(value, list):
             if action is None or isinstance(action, argparse._AppendAction):
                 for list_elem in value:
@@ -965,4 +948,3 @@ ArgumentParser.parse_known = ArgumentParser.parse_known_args
 RawFormatter = RawDescriptionHelpFormatter
 DefaultsFormatter = ArgumentDefaultsHelpFormatter
 DefaultsRawFormatter = ArgumentDefaultsRawHelpFormatter
-
